"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildGetOverridesFromVariants = exports.getOverridesFromVariants = void 0;
/*
  Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.

  Licensed under the Apache License, Version 2.0 (the "License").
  You may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
const typescript_1 = __importStar(require("typescript"));
/**
 * Given a list of style variants, select a given one based on input props
 * @internal
 * @param variants list of style variants to select from
 * @param props variant values to select from the list, may include additional props, to tidy up usage upstream
 */
function getOverridesFromVariants(variants, props) {
    // Get unique keys from the provided variants
    const variantValueKeys = [...new Set(variants.flatMap((variant) => Object.keys(variant.variantValues)))];
    /*
      Get variant value object from provided props,
      dropping keys that aren't in variantValueKeys, or whose vals are falsey
    */
    const variantValuesFromProps = Object.keys(props)
        .filter((i) => variantValueKeys.includes(i) && props[i])
        .reduce((acc, key) => {
        return {
            ...acc,
            [key]: props[key],
        };
    }, {});
    const matchedVariants = variants.filter(({ variantValues }) => {
        return (Object.keys(variantValues).length === Object.keys(variantValuesFromProps).length &&
            Object.entries(variantValues).every(([key, value]) => variantValuesFromProps[key] === value));
    });
    return matchedVariants.reduce((overrides, variant) => {
        return { ...overrides, ...variant.overrides };
    }, {});
}
exports.getOverridesFromVariants = getOverridesFromVariants;
const buildGetOverridesFromVariants = () => [
    typescript_1.factory.createFunctionDeclaration(undefined, [typescript_1.factory.createToken(typescript_1.default.SyntaxKind.ExportKeyword)], undefined, typescript_1.factory.createIdentifier('getOverridesFromVariants'), [typescript_1.factory.createTypeParameterDeclaration(typescript_1.factory.createIdentifier('T'), undefined, undefined)], [
        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('variants'), undefined, typescript_1.factory.createArrayTypeNode(typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('Variant'), undefined)), undefined),
        typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('props'), undefined, typescript_1.factory.createTypeLiteralNode([
            typescript_1.factory.createIndexSignature(undefined, undefined, [
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('key'), undefined, typescript_1.factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.StringKeyword), undefined),
            ], typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('T'), undefined)),
        ]), undefined),
    ], typescript_1.factory.createTypeLiteralNode([
        typescript_1.factory.createIndexSignature(undefined, undefined, [
            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('key'), undefined, typescript_1.factory.createKeywordTypeNode(typescript_1.default.SyntaxKind.StringKeyword), undefined),
        ], typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('Variant'), undefined)),
    ]), typescript_1.factory.createBlock([
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('variantValueKeys'), undefined, undefined, typescript_1.factory.createArrayLiteralExpression([
                typescript_1.factory.createSpreadElement(typescript_1.factory.createNewExpression(typescript_1.factory.createIdentifier('Set'), undefined, [
                    typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('variants'), typescript_1.factory.createIdentifier('flatMap')), undefined, [
                        typescript_1.factory.createArrowFunction(undefined, undefined, [
                            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('variant'), undefined, undefined, undefined),
                        ], undefined, typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('keys')), undefined, [
                            typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('variant'), typescript_1.factory.createIdentifier('variantValues')),
                        ])),
                    ]),
                ])),
            ], false)),
        ], typescript_1.default.NodeFlags.Const)),
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('variantValuesFromProps'), undefined, typescript_1.factory.createTypeReferenceNode(typescript_1.factory.createIdentifier('VariantValues'), undefined), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('keys')), undefined, [typescript_1.factory.createIdentifier('props')]), typescript_1.factory.createIdentifier('filter')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('i'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBinaryExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('variantValueKeys'), typescript_1.factory.createIdentifier('includes')), undefined, [typescript_1.factory.createIdentifier('i')]), typescript_1.factory.createToken(typescript_1.default.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('props'), typescript_1.factory.createIdentifier('i')))),
            ]), typescript_1.factory.createIdentifier('reduce')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('acc'), undefined, undefined, undefined),
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('key'), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createReturnStatement(typescript_1.factory.createObjectLiteralExpression([
                        typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier('acc')),
                        typescript_1.factory.createPropertyAssignment(typescript_1.factory.createComputedPropertyName(typescript_1.factory.createIdentifier('key')), typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('props'), typescript_1.factory.createIdentifier('key'))),
                    ], true)),
                ], true)),
                typescript_1.factory.createObjectLiteralExpression([], false),
            ])),
        ], typescript_1.default.NodeFlags.Const)),
        typescript_1.factory.createVariableStatement(undefined, typescript_1.factory.createVariableDeclarationList([
            typescript_1.factory.createVariableDeclaration(typescript_1.factory.createIdentifier('matchedVariants'), undefined, undefined, typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('variants'), typescript_1.factory.createIdentifier('filter')), undefined, [
                typescript_1.factory.createArrowFunction(undefined, undefined, [
                    typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createObjectBindingPattern([
                        typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('variantValues'), undefined),
                    ]), undefined, undefined, undefined),
                ], undefined, typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                    typescript_1.factory.createReturnStatement(typescript_1.factory.createParenthesizedExpression(typescript_1.factory.createBinaryExpression(typescript_1.factory.createBinaryExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('keys')), undefined, [typescript_1.factory.createIdentifier('variantValues')]), typescript_1.factory.createIdentifier('length')), typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsEqualsEqualsToken), typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('keys')), undefined, [typescript_1.factory.createIdentifier('variantValuesFromProps')]), typescript_1.factory.createIdentifier('length'))), typescript_1.factory.createToken(typescript_1.default.SyntaxKind.AmpersandAmpersandToken), typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('Object'), typescript_1.factory.createIdentifier('entries')), undefined, [typescript_1.factory.createIdentifier('variantValues')]), typescript_1.factory.createIdentifier('every')), undefined, [
                        typescript_1.factory.createArrowFunction(undefined, undefined, [
                            typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createArrayBindingPattern([
                                typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('key'), undefined),
                                typescript_1.factory.createBindingElement(undefined, undefined, typescript_1.factory.createIdentifier('value'), undefined),
                            ]), undefined, undefined, undefined),
                        ], undefined, typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBinaryExpression(typescript_1.factory.createElementAccessExpression(typescript_1.factory.createIdentifier('variantValuesFromProps'), typescript_1.factory.createIdentifier('key')), typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsEqualsEqualsToken), typescript_1.factory.createIdentifier('value'))),
                    ])))),
                ], true)),
            ])),
        ], typescript_1.default.NodeFlags.Const)),
        typescript_1.factory.createReturnStatement(typescript_1.factory.createCallExpression(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('matchedVariants'), typescript_1.factory.createIdentifier('reduce')), undefined, [
            typescript_1.factory.createArrowFunction(undefined, undefined, [
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('overrides'), undefined, undefined, undefined),
                typescript_1.factory.createParameterDeclaration(undefined, undefined, undefined, typescript_1.factory.createIdentifier('variant'), undefined, undefined, undefined),
            ], undefined, typescript_1.factory.createToken(typescript_1.default.SyntaxKind.EqualsGreaterThanToken), typescript_1.factory.createBlock([
                typescript_1.factory.createReturnStatement(typescript_1.factory.createObjectLiteralExpression([
                    typescript_1.factory.createSpreadAssignment(typescript_1.factory.createIdentifier('overrides')),
                    typescript_1.factory.createSpreadAssignment(typescript_1.factory.createPropertyAccessExpression(typescript_1.factory.createIdentifier('variant'), typescript_1.factory.createIdentifier('overrides'))),
                ], false)),
            ], true)),
            typescript_1.factory.createObjectLiteralExpression([], false),
        ])),
    ], true)),
];
exports.buildGetOverridesFromVariants = buildGetOverridesFromVariants;
//# sourceMappingURL=getOverridesFromVariants.js.map